# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.contrib.postgres.fields import JSONField
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .choices import ASSIZE_COURT, CRIMINAL_COURT
from .managers import (
    ArchiveResourceManager,
    BiblioResourceManager,
    JobManager,
    ResourcesGroupManager,
    StatisticManager,
)


@python_2_unicode_compatible
class Institution(models.Model):
    '''
    This represents an Institution. (Main object of this app)
    Through foreignkeys, an institution can have:
        - images
        - videos
        - various links
        - different locations through years
        - various role through years
        - various events registered
        - people who work in it
        - statistics
    '''
    name = models.CharField(_("Name"), max_length=255)
    created_at = models.DateField(_("Created at"), auto_now_add=True)
    contributors = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Contributors"))
    locations = models.ManyToManyField(
        "Location",
        verbose_name=_("Location"),
        blank=True)
    construction_date = models.CharField(
        _("Construction Date"),
        max_length=255,
        blank=True)
    opening_date = models.CharField(
        _("Opening Date"),
        max_length=255,
        blank=True)
    closing_date = models.CharField(
        _("Closing Date"),
        max_length=255,
        blank=True)
    particularities = models.TextField(_("Particularities"), blank=True)
    merimee_reference = models.CharField(
        _("Mérimée Reference"),
        max_length=255,
        blank=True)
    resources_group = models.ForeignKey(
        "ResourcesGroup",
        verbose_name="Resources Group")
    roles = models.ManyToManyField("Role", verbose_name='Role')
    latest_type = JSONField(
        _("Latest Type"),
        blank=True,
        null=True,
        max_length=255,
        editable=False)

    class Meta:
        verbose_name = _("Institution")
        verbose_name_plural = _("Institutions")

    def __str__(self):
        return self.name

    def get_latest_location_dict_for_map(self):
        if not self.locations.all():
            return
        location = self.locations.all()[::-1][0]
        return {
            "title": self.name,
            "address": location.get_full_address,
            "lat": location.latitude,
            "lng": location.longitude,
            "detailUrl": reverse(
                'institution_detail', kwargs={"pk": self.pk})
        }

    def update_latest_type(self):
        if self.roles.all():
            latest_role = self.roles.latest()
            self.latest_type = latest_role.institution_type
            self.save()
        else:
            self.latest_type = ["undefined"]

    @property
    def display_contributors(self):
        contributors_names = [
            p.get_full_name() if p.get_full_name() else p.username
            for p in self.contributors.all()
        ]
        return ', '.join(contributors_names) or _("N/A")


@python_2_unicode_compatible
class JurisdictionArea(models.Model):
    institution = models.ForeignKey("Institution", verbose_name="Institution")
    area = models.CharField(_("Area"), max_length=255)
    date_range_display = models.CharField(
        _("Date Range Display"),
        blank=True,
        max_length=255,
        help_text=_("The displayed date"))
    date_start = models.DateField(
        _("Date Start"),
        blank=True,
        help_text=_("The date used for sorting"))

    class Meta:
        verbose_name = _("Jurisdiction Area")
        verbose_name_plural = _("Jurisdiction Areas")
        ordering = ['date_start']

    def __str__(self):
        return _("Area {area}").format(area=self.area)


@python_2_unicode_compatible
class Location(models.Model):
    address = models.CharField(_("Address"), max_length=255)
    city = models.CharField(_("City"), max_length=255)
    county = models.CharField(_("County"), max_length=255, blank=True)
    country = models.CharField(_("Country"), max_length=255)
    insee_id = models.CharField(_("INSEE id"), max_length=255, blank=True)
    date_start = models.DateField(
        _("Date Start"),
        blank=True,
        help_text=_("The date used for sorting"))
    date_start_display = models.CharField(
        _("Date Start Display"),
        max_length=255,
        blank=True,
        help_text=_("The displayed date"))
    date_end = models.DateField(
        _("Date End"),
        blank=True,
        null=True,
        help_text=_("The date used for sorting"))
    date_end_display = models.CharField(
        _("Date End Display"),
        max_length=255,
        blank=True,
        help_text=_("The date displayed"))
    longitude = models. FloatField(_("Longitude"))
    latitude = models. FloatField(_("Latitude"))

    class Meta:
        verbose_name = _("Location")
        verbose_name_plural = _("Locations")
        ordering = ['date_start']
        get_latest_by = 'date_start'

    def __str__(self):
        return self.get_full_address or "Unknown Address"

    @property
    def get_full_address(self):
        address_as_list = [self.address, self.city, self.county, self.country]
        return ", ".join([el for el in address_as_list if el])


@python_2_unicode_compatible
class ResourcesGroup(models.Model):
    crimino_resource = models.ForeignKey(
        "CriminoResource",
        verbose_name="Crimino Resource")
    bibliography_resource = models.ForeignKey(
        "BiblioResource",
        verbose_name="Bibliography Resource")
    archive_resource = models.ForeignKey(
        "ArchiveResource",
        verbose_name="Archive Resource")
    miscellaneous_urls = models.ManyToManyField(
        "Link",
        verbose_name="Miscellaneous URLs",
        blank=True)

    objects = ResourcesGroupManager()

    class Meta:
        verbose_name = _("Resources Group")
        verbose_name_plural = _("Resources Groups")

    def __str__(self):
        return _("Resources Group {pk}").format(pk=self.pk)


@python_2_unicode_compatible
class BiblioResource(models.Model):
    criminocorpus_url = models.URLField(_("CriminoCorpus URL"), blank=True)
    bnf_url = models.URLField(_("BnF URL"), blank=True)
    crhcp_url = models.URLField(_("CRHCP URL"), blank=True)
    other_urls = models.ManyToManyField(
        "Link",
        verbose_name=_("Other URLs"),
        blank=True)
    other = models.TextField(
        _("Other"),
        blank=True,
        help_text=_("Complementary resources (could be offline)"))

    objects = BiblioResourceManager()

    class Meta:
        verbose_name = _("Bibliography Resource")
        verbose_name_plural = _("Bibliography Resources")

    def __str__(self):
        return _("Biblio Resource {pk}").format(pk=self.pk)


@python_2_unicode_compatible
class ArchiveResource(models.Model):
    crhcp_url = models.URLField(_("CRHCP URL"), blank=True)
    gallica_url = models.URLField(_("Gallica URL"), blank=True)
    other_urls = models.ManyToManyField(
        "Link",
        verbose_name=_("Other URLs"),
        blank=True)
    other = models.TextField(
        _("Other"),
        blank=True,
        help_text=_("Complementary resources (could be offline)"))

    objects = ArchiveResourceManager()

    class Meta:
        verbose_name = _("Archive Resource")
        verbose_name_plural = _("Archive Resources")

    def __str__(self):
        return _("Archive Resource {pk}").format(pk=self.pk)


@python_2_unicode_compatible
class CriminoResource(models.Model):
    criminocorpus_url = models.URLField(_("CriminoCorpus URL"), blank=True)
    other = models.TextField(
        _("Other"),
        blank=True,
        help_text=_("Complementary resources (could be offline)"))

    class Meta:
        verbose_name = _("Crimino Resource")
        verbose_name_plural = _("Crimino Resources")

    def __str__(self):
        return _("Crimino Resource {pk}").format(pk=self.pk)


@python_2_unicode_compatible
class Link(models.Model):
    title = models.CharField(_("Title"), max_length=255)
    url = models.URLField(_("URL"))

    class Meta:
        verbose_name = _("Link")
        verbose_name_plural = _("Links")

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Role(models.Model):
    description = models.TextField(_("Description"))
    institution_type = JSONField(
        _("Type"),
        max_length=255,
        null=True,
        blank=True)
    prisoner_count = models.PositiveIntegerField(
        _("Prisoner Count"),
        null=True,
        blank=True)
    prisoner_gender = JSONField(_(
        "Prisoner Gender"),
        max_length=50,
        null=True,
        blank=True)
    prisoner_age_category = JSONField(
        _("Prisoner Age Category"),
        max_length=50,
        null=True,
        blank=True)
    prisoner_type = JSONField(
        _("Prisoner Type"),
        max_length=255,
        null=True,
        blank=True)
    date_start = models.DateField(
        _("Date Start"),
        help_text=_("The date used for sorting"))
    date_start_display = models.CharField(
        _("Date Start Display"),
        max_length=255,
        blank=True,
        help_text="The date displayed")
    date_end = models.DateField(
        _("Date End"),
        blank=True,
        null=True,
        help_text="The date used for sorting")
    date_end_display = models.CharField(
        _("Date End Display"),
        max_length=255,
        blank=True,
        help_text=_("The displayed date"))

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")
        ordering = ['date_start']
        get_latest_by = 'date_start'

    def __str__(self):
        return _("Role: {description}").format(description=self.description)

    @property
    def has_prisoner(self):
        return (ASSIZE_COURT not in self.institution_type and
                CRIMINAL_COURT not in self.institution_type)


@python_2_unicode_compatible
class Event(models.Model):
    institution = models.ForeignKey("Institution", verbose_name="Institution")
    name = models.CharField(
        _("Name"),
        max_length=255,
        help_text=_("Ex: Fire in Saint-Lô Prison"))
    event_type = models.CharField(_("Type"), max_length=255)
    description = models.TextField(_("Description"), blank=True)
    datetime_start = models.DateTimeField(
        _("DateTime Start"),
        blank=True,
        help_text=_("The date used for sorting"))
    datetime_start_display = models.CharField(
        _("DateTime Start Display"),
        max_length=255,
        blank=True,
        help_text=_("The displayed date"))
    datetime_end = models.DateTimeField(
        _("DateTime End"),
        blank=True,
        null=True,
        help_text=_("The date used for sorting"))
    datetime_end_display = models.CharField(
        _("DateTime End Display"),
        max_length=255,
        blank=True,
        help_text=_("The displayed date"))

    class Meta:
        verbose_name = _("Event")
        verbose_name_plural = _("Events")
        ordering = ['datetime_start', ]

    def __str__(self):
        return _("Event {name}").format(name=self.name)


@python_2_unicode_compatible
class Image(models.Model):
    institutions = models.ManyToManyField(
        "Institution",
        related_name="images",
        verbose_name="Institutions")
    name = models.CharField(_("Name"), max_length=255)
    image = models.ImageField(_("Image"), upload_to="img/%Y/%m/%d/")

    class Meta:
        verbose_name = _("Image")
        verbose_name_plural = _("Images")

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Video(models.Model):
    institutions = models.ManyToManyField(
        "Institution",
        related_name="videos",
        verbose_name="Institutions")
    name = models.CharField(_("Name"), max_length=255)
    video = models.FileField(_("Video"), upload_to="video/%Y/%m/%d/")

    class Meta:
        verbose_name = _("Video")
        verbose_name_plural = _("Videos")

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Statistic(models.Model):
    institution = models.ForeignKey("Institution", verbose_name="Institution")
    online_stats_name = models.CharField(
        _("Online Stats Name"),
        max_length=255,
        blank=True)
    online_stats_url = models.URLField(_("Online Stats URL"), blank=True)
    offline_stats_name = models.CharField(
        _("Offline Stats Name"),
        blank=True,
        max_length=255)
    offline_stats_reference = models.TextField(
        _("Offline Stats Reference"),
        blank=True)
    offline_stats_data = models.TextField(_("Offline Stats data"), blank=True)
    graphs = models.ManyToManyField(
        "Graph",
        verbose_name=_("Graphs"),
        blank=True)

    objects = StatisticManager()

    class Meta:
        verbose_name = _("Statistic")
        verbose_name_plural = _("Statistics")

    def __str__(self):
        return _("Statistics for {institution}").format(
            institution=self.institution.name)


@python_2_unicode_compatible
class Job(models.Model):
    institution = models.ForeignKey("Institution", verbose_name="Institution")
    person = models.ForeignKey("Person", verbose_name="Person")
    date_range = models.CharField(_("Date Range"), max_length=255, blank=True)
    date_start = models.DateField(
        _("Date Start"),
        help_text="The date used for sorting")
    role = models.CharField(_("Role"), max_length=255)

    objects = JobManager()

    class Meta:
        verbose_name = _("Job")
        verbose_name_plural = _("Jobs")
        ordering = ['date_start']

    def __str__(self):
        return _("Role: {role} for {person}").format(
            role=self.role, person=self.person.full_name)


@python_2_unicode_compatible
class Person(models.Model):
    first_name = models.CharField(_("First Name"), max_length=255, blank=True)
    last_name = models.CharField(_("Last Name"), max_length=255)
    birthday = models.CharField(_("Birthday"), max_length=255, blank=True)
    deathday = models.CharField(_("Date of Death"), max_length=255, blank=True)

    class Meta:
        verbose_name = _("Person")
        verbose_name_plural = _("Persons")

    def __str__(self):
        return self.full_name or _("Missing Name")

    @property
    def full_name(self):
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()


@python_2_unicode_compatible
class Graph(models.Model):
    name = models.CharField(_("Name"), max_length=255)
    graph = models.ImageField(_("Graph"), upload_to="graph/%Y/%m/%d/")

    class Meta:
        verbose_name = _("Graph")
        verbose_name_plural = _("Graphs")

    def __str__(self):
        return self.name
