import factory
import faker

from datetime import datetime, timedelta
from random import randint

from django.template.defaultfilters import slugify

from ..models import (
    ArchiveResource,
    BiblioResource,
    CriminoResource,
    Event,
    Graph,
    Institution,
    Image,
    JurisdictionArea,
    Job,
    Link,
    Location,
    Person,
    ResourcesGroup,
    Role,
    Statistic,
    Video,
)


faker = faker.Factory.create()


class UserFactory(factory.django.DjangoModelFactory):
    first_name = factory.lazy_attribute(lambda o: faker.first_name())
    last_name = factory.lazy_attribute(lambda o: faker.last_name())
    username = factory.lazy_attribute(
        lambda o: slugify(o.first_name + '.' + o.last_name))
    email = factory.lazy_attribute(lambda o: o.username + "@example.com")

    class Meta:
        model = 'auth.User'
        django_get_or_create = ('username', )

    @factory.lazy_attribute
    def date_joined(self):
        return datetime.now() - timedelta(days=randint(5, 50))

    last_login = factory.lazy_attribute(
        lambda o: o.date_joined + timedelta(days=4))


class InstitutionFactory(factory.django.DjangoModelFactory):
    name = factory.Sequence(lambda n: 'Institution %03d' % n)
    resources_group = factory.SubFactory(
        'crimino_jail.tests.factories.ResourcesGroupFactory')

    class Meta:
        model = Institution

    @factory.post_generation
    def locations(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of locations were passed in, use them
            for location in extracted:
                self.locations.add(location)

    @factory.post_generation
    def roles(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of roles were passed in, use them
            for role in extracted:
                self.roles.add(role)


class InstitutionObjectBaseFactory(factory.django.DjangoModelFactory):
    institution = factory.SubFactory(
        'crimino_jail.tests.factories.InstitutionFactory')


class JurisdictionAreaFactory(InstitutionObjectBaseFactory):
    date_start = datetime(2000, 01, 01)

    class Meta:
        model = JurisdictionArea


class LocationFactory(factory.django.DjangoModelFactory):
    date_start = datetime(2000, 01, 01)
    longitude = 10
    latitude = 3.42

    class Meta:
        model = Location


class VideoFactory(factory.django.DjangoModelFactory):
    video = factory.django.FileField(filename='test_file.garbage')

    @factory.post_generation
    def institutions(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of institutions were passed in, use them
            for institution in extracted:
                self.institutions.add(institution)

    class Meta:
        model = Video


class ImageFactory(factory.django.DjangoModelFactory):
    image = factory.django.ImageField(filename='test_file.garbage')

    @factory.post_generation
    def institutions(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of institutions were passed in, use them
            for institution in extracted:
                self.institutions.add(institution)

    class Meta:
        model = Image


class EventFactory(InstitutionObjectBaseFactory):
    name = factory.Sequence(lambda n: 'Institution %03d' % n)
    datetime_start = datetime(2010, 1, 1)

    class Meta:
        model = Event


class RoleFactory(factory.django.DjangoModelFactory):
    date_start = datetime(2010, 1, 1)

    class Meta:
        model = Role


class ResourcesGroupFactory(factory.django.DjangoModelFactory):
    bibliography_resource = factory.SubFactory(
        'crimino_jail.tests.factories.BiblioResourceFactory')
    archive_resource = factory.SubFactory(
        'crimino_jail.tests.factories.ArchiveResourceFactory')
    crimino_resource = factory.SubFactory(
        'crimino_jail.tests.factories.CriminoResourceFactory')

    class Meta:
        model = ResourcesGroup


class BiblioResourceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BiblioResource


class ArchiveResourceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ArchiveResource


class CriminoResourceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CriminoResource


class LinkFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Link


class StatisticFactory(InstitutionObjectBaseFactory):
    class Meta:
        model = Statistic


class GraphFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Graph

    @factory.post_generation
    def statistics(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of statistics were passed in, use them
            for statistic in extracted:
                self.statistics.add(statistic)


class JobFactory(InstitutionObjectBaseFactory):
    person = factory.SubFactory('crimino_jail.tests.factories.PersonFactory')
    date_start = datetime(2010, 1, 1)

    class Meta:
        model = Job


class PersonFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Person
