from datetime import datetime

from django.test import SimpleTestCase, TestCase

from .factories import (
    EventFactory,
    InstitutionFactory,
    LocationFactory,
    RoleFactory,
    JurisdictionAreaFactory,
    JobFactory,
    UserFactory,
)
from ..choices import CRIMINAL_COURT
from ..models import (
    ArchiveResource,
    BiblioResource,
    CriminoResource,
    Event,
    Graph,
    Image,
    Institution,
    Link,
    Location,
    Role,
    JurisdictionArea,
    Job,
    Person,
    ResourcesGroup,
    Statistic,
    Video,
)


class TestOrdering(TestCase):
    @classmethod
    def setUpTestData(self):
        self.date2010 = datetime(2010, 01, 01)
        self.date2011 = datetime(2011, 01, 01)
        self.date2012 = datetime(2012, 01, 01)

    def test_ordering_event(self):
        self.event3 = EventFactory(datetime_start=self.date2012)
        self.event2 = EventFactory(datetime_start=self.date2011)
        self.event1 = EventFactory(datetime_start=self.date2010)
        events = Event.objects.all()
        assert events[0] == self.event1
        assert events[2] == self.event3

    def test_ordering_location(self):
        self.location2 = LocationFactory(date_start=self.date2011)
        self.location1 = LocationFactory(date_start=self.date2010)
        locations = Location.objects.all()
        assert locations[0] == self.location1
        assert locations[1] == self.location2

    def test_ordering_role(self):
        self.role2 = RoleFactory(date_start=self.date2011)
        self.role1 = RoleFactory(date_start=self.date2010)
        roles = Role.objects.all()
        assert roles[0] == self.role1
        assert roles[1] == self.role2

    def test_ordering_jurisdiction_area(self):
        self.jurisdictionarea2 = JurisdictionAreaFactory(
            date_start=self.date2011)
        self.jurisdictionarea1 = JurisdictionAreaFactory(
            date_start=self.date2010)
        jurisdiction_areas = JurisdictionArea.objects.all()
        assert jurisdiction_areas[0] == self.jurisdictionarea1
        assert jurisdiction_areas[1] == self.jurisdictionarea2

    def test_ordering_job(self):
        self.job2 = JobFactory(date_start=self.date2011)
        self.job1 = JobFactory(date_start=self.date2010)
        jobs = Job.objects.all()
        assert jobs[0] == self.job1
        assert jobs[1] == self.job2


class TestStaticModels(SimpleTestCase):
    """
    Here are dummy test for method which does not require the db
    For example: the __str__ method for all models

    Note: we're using a SimpleTestCase --> No db access is required.
      We call models with: `MyModel()` instead of `MyModel.objects.create()`
    """
    def test_institution_str(self):
        institution = Institution(name="foo")
        assert institution.__str__() == "foo"

    def test_jurisdiction_area_str(self):
        jurisdiction_area = JurisdictionArea(area="foo")
        assert jurisdiction_area.__str__() == "Area foo"

    def test_location_str(self):
        location = Location(address="foo", city="bar")
        assert location.__str__() == "foo, bar"

        location = Location()
        assert location.__str__() == "Unknown Address"

    def test_resource_group_str(self):
        resources_group = ResourcesGroup()
        assert resources_group.__str__() == "Resources Group None"

    def test_archive_resource_str(self):
        resource = ArchiveResource()
        assert resource.__str__() == "Archive Resource None"

    def test_biblio_resource_str(self):
        resource = BiblioResource()
        assert resource.__str__() == "Biblio Resource None"

    def test_crimino_resource_str(self):
        resource = CriminoResource()
        assert resource.__str__() == "Crimino Resource None"

    def test_link_str(self):
        link = Link(title="foo")
        assert link.__str__() == 'foo'

    def test_role_str(self):
        role = Role(description="bar")
        assert role.__str__() == "Role: bar"

    def test_event_str(self):
        event = Event(name="foo")
        assert event.__str__() == "Event foo"

    def test_image_str(self):
        img = Image(name="foo")
        assert img.__str__() == "foo"

    def test_video_str(self):
        video = Video(name="video")
        assert video.__str__() == "video"

    def test_statistic_str(self):
        institution = Institution(name="foo")
        statistic = Statistic(institution=institution)
        assert statistic.__str__() == "Statistics for foo"

    def test_job_str(self):
        person = Person(first_name="bar", last_name="baz")
        job = Job(role="foo", person=person)
        assert job.__str__() == "Role: foo for bar baz"

    def test_person_str(self):
        person = Person(first_name="bar", last_name="baz")
        assert person.__str__() == "bar baz"

        person = Person(last_name="baz")
        assert person.__str__() == "baz"

        person = Person(first_name="bar")
        assert person.__str__() == "bar"

        person = Person()
        assert person.__str__() == "Missing Name"

    def test_graph_str(self):
        graph = Graph(name="foo")
        assert graph.__str__() == "foo"

    def test_role_has_prisoner(self):
        role = Role(institution_type=CRIMINAL_COURT)
        assert role.has_prisoner is False

        role = Role(institution_type="zbla")
        assert role.has_prisoner is True


class TestInstitution(TestCase):
    @classmethod
    def setUpTestData(self):
        self.institution = InstitutionFactory()

    def test_display_contributors(self):
        assert self.institution.display_contributors == "N/A"

        user1 = UserFactory(first_name="foo")
        user2 = UserFactory()
        user3 = UserFactory(first_name="", last_name="", username="bar")

        self.institution.contributors.add(user1)
        self.institution.contributors.add(user2)
        self.institution.contributors.add(user3)
        contributors = self.institution.display_contributors
        for string in ['foo', ',', 'bar']:
            assert string in contributors

    def test_get_latest_location_dict_for_map(self):
        d = self.institution.get_latest_location_dict_for_map()

        self.institution.locations.add(LocationFactory())
        self.institution.locations.add(LocationFactory())

        d = self.institution.get_latest_location_dict_for_map()
        keys = ["title", "address", "lat", "lng", "detailUrl"]
        for key in keys:
            assert key in d


class TestLocation(TestCase):
    def test_get_latest(self):
        LocationFactory()
        LocationFactory()
        d = LocationFactory(date_start=datetime(2016, 1, 1))

        d2 = Location.objects.latest()
        assert d == d2
