from django.core.urlresolvers import reverse_lazy
from django.test import RequestFactory, TestCase

from .factories import (
    EventFactory,
    InstitutionFactory,
    ImageFactory,
    JobFactory,
    JurisdictionAreaFactory,
    LocationFactory,
    PersonFactory,
    RoleFactory,
)
from ..views import (
    EventDetailView,
    InstitutionDetailView,
    InstitutionListView,
    PersonDetailView,
)


class TestView(TestCase):
    @classmethod
    def setUpTestData(self):
        self.factory = RequestFactory()


class TestPersonDetailView(TestView):
    @classmethod
    def setUpTestData(self):
        super(TestPersonDetailView, self).setUpTestData()
        self.person = PersonFactory()
        self.url = reverse_lazy('person_detail',
                                kwargs={"pk": self.person.pk})

    def test_template(self):
        with self.assertTemplateUsed('person_detail.html'):
            request = self.factory.get(self.url)
            resp = PersonDetailView.as_view()(request, pk=self.person.pk)
            resp.render()

    def test_queries(self):
        with self.assertNumQueries(2):
            self.client.get(self.url)


class TestInstitutionDetailView(TestView):
    @classmethod
    def setUpTestData(self):
        super(TestInstitutionDetailView, self).setUpTestData()
        self.institution = InstitutionFactory()
        self.url = reverse_lazy(
            'institution_detail',
            kwargs={"pk": self.institution.pk})

    def build_institution_related_models(self):
        JobFactory(institution=self.institution)
        JobFactory(institution=self.institution)
        EventFactory(institution=self.institution)
        EventFactory(institution=self.institution)
        ImageFactory.create(institutions=(self.institution, ))
        ImageFactory.create(institutions=(self.institution, ))
        role1 = RoleFactory()
        role2 = RoleFactory()
        self.institution.roles.add(role1)
        self.institution.roles.add(role2)
        location = LocationFactory()
        self.institution.locations.add(location)
        JurisdictionAreaFactory(institution=self.institution)

    def test_template(self):
        with self.assertTemplateUsed('institution_detail.html'):
            request = self.factory.get(self.url)
            response = InstitutionDetailView.as_view()(
                request, pk=self.institution.pk)
            response.render()

    def test_get_context_data(self):
        response = self.client.get(self.url)
        assert 'markers_location' in response.context

    def test_queries(self):
        self.build_institution_related_models()
        with self.assertNumQueries(17):
            self.client.get(self.url)


class TestEventDetailView(TestView):
    @classmethod
    def setUpTestData(self):
        super(TestEventDetailView, self).setUpTestData()
        self.event = EventFactory()
        self.url = reverse_lazy('event_detail', kwargs={"pk": self.event.pk})

    def test_template(self):
        with self.assertTemplateUsed('event_detail.html'):
            request = self.factory.get(self.url)
            response = EventDetailView.as_view()(request, pk=self.event.pk)
            response.render()

    def test_queries(self):
        with self.assertNumQueries(1):
            self.client.get(self.url)


class TestInstitutionListView(TestView):
    @classmethod
    def setUpTestData(self):
        super(TestInstitutionListView, self).setUpTestData()
        self.institution = InstitutionFactory()
        self.url = reverse_lazy('institution_list')

    def test_template(self):
        with self.assertTemplateUsed('institution_list.html'):
            request = self.factory.get(self.url)
            response = InstitutionListView.as_view()(request)
            response.render()

    def test_queries(self):
        with self.assertNumQueries(2):
            self.client.get(self.url)

    def test_context_data(self):
        response = self.client.get(self.url)
        assert 'markerClusterImageUrl' in response.context
        assert 'locations' in response.context
