import json

from django.contrib.staticfiles.templatetags.staticfiles import static
from django.views.generic import DetailView, TemplateView

from .models import Event, Institution, Location, Person
from .utils import merge_dict


class InstitutionDetailView(DetailView):
    model = Institution
    template_name = "institution_detail.html"
    context_object_name = "institution"
    prefetch_related = [
        'contributors',
        'event_set',
        'job_set',
        'jurisdictionarea_set',
        'locations',
        'roles',
        'statistic_set',
        'images',
        'videos',
    ]
    related_fields = ['resources_group']

    def get_queryset(self):
        queryset = super(InstitutionDetailView, self).get_queryset()
        queryset = queryset\
            .select_related(*self.related_fields)\
            .prefetch_related(*self.prefetch_related)
        return queryset

    def get_context_data(self, **kwargs):
        context = super(InstitutionDetailView, self).get_context_data(**kwargs)
        locations = Location.objects.filter(institution=self.object.pk)
        markers = [
            {
                "title": self.object.name,
                "address": loc.get_full_address,
                "lat": loc.latitude,
                "lng": loc.longitude
            }
            for loc in locations]
        context['markers_location'] = json.dumps(markers)
        return context


class PersonDetailView(DetailView):
    model = Person
    template_name = "person_detail.html"
    context_object_name = "person"

    def get_queryset(self):
        return super(PersonDetailView, self).get_queryset()\
            .prefetch_related('job_set')


class EventDetailView(DetailView):
    model = Event
    template_name = "event_detail.html"
    context_object_name = "event"

    def get_queryset(self):
        return super(EventDetailView, self).get_queryset()\
            .select_related('institution')


class InstitutionListView(TemplateView):
    """
    This is intended the view is just a template view. Our aim is to list
    and present institutions but the layout use google map to list them
    all. That's why a template view is good enough.
    We need to send institutions with their latest locations.
    """
    template_name = "institution_list.html"

    def get_context_data(self, **kwargs):
        context = super(InstitutionListView, self).get_context_data(**kwargs)

        context["markerClusterImageUrl"] = static('map/m1.png')[:-5]

        institutions = Institution.objects.all().prefetch_related("locations")
        # Create an array of alphabetical characters used to label the markers.
        labels = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        locations = [
            merge_dict(
                institution.get_latest_location_dict_for_map(),
                {"label": labels[i % len(labels)]}
            )
            for i, institution in enumerate(institutions)
            if institution.get_latest_location_dict_for_map()]
        context['locations'] = json.dumps(locations)

        return context
